
/**
 *	S3C (Simple Server-side Cache)
 *	Server-side Object Caching Framework 
 *
 *	Copyright (C) 2005  Kamran Zafar
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	Contact Info:
 *	Email: xeus.man@gmail.com
 */

package xeus.s3c.xml.parser;

import java.io.IOException;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

import xeus.s3c.Cache;
import xeus.s3c.CacheManager;
import xeus.s3c.Slot;
import xeus.s3c.exception.SlotNotAvailableException;

/**
 * @author Kamran Zafar
 *  
 */
public class CacheXMLReader extends DefaultHandler {

    private boolean cache;
    private boolean bound;
    private boolean slot;
    private Cache s3c;

    public CacheXMLReader()
    {
        super();
        cache=false;
        slot=false;
        bound=false;
    }
    
    public void read(String fileName) throws IOException,SAXException {
        XMLReader readerObj = XMLReaderFactory
                .createXMLReader("org.apache.xerces.parsers.SAXParser");
        
        readerObj.setFeature("http://xml.org/sax/features/validation", true);
        readerObj.setContentHandler(this);
        readerObj.setErrorHandler(this);
        readerObj.parse(fileName);
    }
    
    public void endElement(String uri, String localName, String qName)
            throws SAXException {
        if (qName.equals("cache")) {
            if (cache)
            {
                cache = false;
                bound=false;
            }
            else
                throw new SAXException("Misplaced end element: " + qName);
        } else if (qName.equals("slot")) {
            if (slot && bound && cache)
                slot = false;
            else
                throw new SAXException("Misplaced end element: " + qName);
        }
    }

    public void startElement(String uri, String localName, String qName,
            Attributes attr) throws SAXException {
        if (qName.equals("cache")) {
            cache = true;
            int timeout;
            int nSlots;

            if(attr.getValue("name")==null)
               throw new SAXException("Cache name attribute is missing");
            
            System.out.println("Cache Name: "+(String)attr.getValue("name"));
            
            if (attr.getValue("timeout") == null) {
                timeout = -1;
            } else {
                try {
                    timeout = Integer.parseInt(attr.getValue("timeout"));
                    System.out.println("Cache Timeout: "+timeout);
                } catch (NumberFormatException e) {
                    throw new SAXException("Timeout value should be numeric: "
                            + attr.getValue("timeout"));
                }
            }
            
            if (attr.getValue("slots") == null) {
                nSlots = 10;
            } else {
                try {
                    nSlots = Integer.parseInt(attr.getValue("slots"));
                    System.out.println("Slots: "+nSlots);
                } catch (NumberFormatException e) {
                    throw new SAXException(
                            "Slots attribute should be numeric: "
                                    + attr.getValue("slots"));
                }
            }
            
            try {
                s3c=CacheManager.create(attr.getValue("name"),timeout,nSlots);
                bound=true;
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        
        else if(qName.equals("slot"))
        {
            if(!(bound || cache))
                throw new SAXException("Slots attribute not allowed here");
            
            System.out.println("Slot Name: "+(String)attr.getValue("name"));
            
            slot=true;
            
            int timeout;
            int nSlots;

            if(attr.getValue("name")==null)
               throw new SAXException("Slot name attribute is missing");
            
            if (attr.getValue("timeout") == null)
                timeout = -1;
            else {
                try {
                    timeout = Integer.parseInt(attr.getValue("timeout"));
                    System.out.println("Slot Timeout: "+timeout);
                } catch (NumberFormatException e) {
                    throw new SAXException("Timeout value should be numeric: "
                            + attr.getValue("timeout"));
                }
            }
            
            Slot s=null;
            try {
                s = s3c.register(attr.getValue("name"));
            } catch (SlotNotAvailableException e) {
                e.printStackTrace();
                return;
            }
            s.setTimeout(timeout);
            s.initialize();
        }
    }
    
    public Cache getCache()
    {
        return s3c;
    }
    
    public void warning(SAXParseException exception) throws SAXException {
        System.err.println("Warning in " + exception.getSystemId()
                + " at line " + exception.getLineNumber() + ", column "
                + exception.getColumnNumber()+" "+exception.getMessage());
    }

    public void error(SAXParseException exception) throws SAXException {
        System.err.println("Error in " + exception.getSystemId() + " at line "
                + exception.getLineNumber() + ", column "
                + exception.getColumnNumber()+" "+exception.getMessage());
    }

    public void fatalError(SAXParseException exception) throws SAXException {
        System.err.println("Fatal error in " + exception.getSystemId()
                + " at line " + exception.getLineNumber() + ", column "
                + exception.getColumnNumber()+" "+exception.getMessage());
        throw exception;
    }
}